//
//								PLOT
// 1D visualization program for multicolumn data stored in text files (ASCII format)
//
// Copyright 2011-2015 by Yuri Ganshin, CMI, University of Wyoming, Laramie.
// http://www.uwyo.edu/cmi/
//
// This program is free sofware released using the Fast Light Tool Kit (FLTK).
// For conditions of distribution and use, see the license agreement in 
//     http://www.fltk.org/COPYING.php
// As a general rule, "free software" means that the user have the freedom
// to run, copy, distribute, study, modify and improve the software.
//
// Please report all bugs and problems on the following page:
// http://www.uwyo.edu/cmi/
//     
#include <cstdio>
#include <cstdlib>
#include <cstring>
#include <iostream>
//
#include <FL/Fl.H>
#include <FL/Fl_Window.H>
#include <FL/Fl_Double_Window.H>
#include <FL/Fl_Scroll.H>
#include <FL/fl_draw.H>	
#include <FL/Fl_Menu_Bar.H>
#include <FL/Fl_Native_File_Chooser.H>
#include <FL/Fl_Int_Input.H>
#include <FL/Fl_Spinner.H>
#include <FL/Fl_Input.H>
#include <FL/Fl_Float_Input.H>
#include <FL/Fl_Input_Choice.H>
#include <FL/Fl_Check_Button.H>
#include <FL/fl_show_colormap.H>
#include <FL/Fl_Output.H>
#include <FL/Fl_Button.H>
#include <FL/Fl_Box.H>
#include <FL/fl_message.H>
#include <FL/fl_ask.H> // for fl_alert() and fl_message()
#include <math.h>

using namespace std;

#define MIN_CURVE	5		// Minimum number of curves (WITH DEPTH) to display
#define MAX_CURVE	60
#define RIM			10		// RIM == border for canvas
#define MARGIN		75
#define FLT_MAX	1E+37
#define FLT_MIN	-1E+37
#define PALSZ		11
#define HIST_INT	100
//
////////////////////////////////////////////////////////////////
/****               List of GLOBAL variables               ****/
FILE *in_fp = NULL;
char inpname[200];
int chosen=-1, line_draw=0, reverseY=0;	
int Ncur = 1;			// Actual number of curves in file (2 and more)
int Ndat = 1;			// Number of samples in a curve
int win_w, win_h;
//
float xticks = 1.0, yticks = 1.0;
float *logs, *mini, *maxi;		// Min-Max for all curves
float min_x, max_x, min_y, max_y;		// for displayed curve
char curve[MAX_CURVE][4], c_word[4], cgbuf[90];
int curve_x=0, curve_y=1, pt_size=5;	
const char *Xtext_hdr = "        X        ";
const char *Ytext_hdr = "        Y        ";
Fl_Color bg_color = FL_WHITE;
Fl_Color pt_color = 1;
//
////////////////////////////////////////////////////////////////
/****            List of Function Prototypes               ****/

void read_file_info(const char *name);

////////////////////////////////////////////////////////////////
/****         Global pointers to the GUI objects           ****/
////////////////////////////////////////////////////////////////

Fl_Double_Window *draw_window;
Fl_Window *opts_window;
Fl_Float_Input *min_x_inp, *max_x_inp, *min_y_inp, *max_y_inp;
Fl_Float_Input *xtick_inp, *ytick_inp;
Fl_Button *apply, *end_this, *choose_color;
Fl_Input *Xaxis_hdr,*Yaxis_hdr;
Fl_Check_Button *connected,*plot_dir;

/////////////////////////////////////////////////////////////////////
/****          List of Widgets and Drawing functions            ****/
/////////////////////////////////////////////////////////////////////

/**********     File-choosing Menu    ************/
class File_menu : public Fl_Window {
	Fl_Native_File_Chooser *fc;
	
	void read_data(const char *filename)	// Process the chosen file
	{
		strcpy(inpname,filename);
		chosen=1;			// will wait while(chosen<0)
	}
	
	// Handle an 'Open' request from the menu
	static void open_cb(Fl_Widget *w, void *v) {
		File_menu *choice = (File_menu*)v;
		choice->fc->title("Open");
		choice->fc->type(Fl_Native_File_Chooser::BROWSE_FILE);
		switch ( choice->fc->show() ) {
			case -1: break;	// Error
			case  1: break; 	// Cancel
			default:		// choice->fc->show() returns 0
				choice->read_data(choice->fc->filename());
				// choice->deactivate();				
				break;
		}
	}
	
	static void quit_cb(Fl_Widget *w, void *v) {
		exit(0);
	}
	
public:
	File_menu() : Fl_Window(500,300,350,150,"Choose the file") {
		Fl_Menu_Bar *menu = new Fl_Menu_Bar(0,0,350,25);
		menu->add("&File/&Open",  FL_COMMAND+'o', open_cb, (void*)this);
		menu->add("&File/&Quit",  FL_COMMAND+'q', quit_cb);
		
		// Describe the chooser
		Fl_Box *box = new Fl_Box(20,45,w()-40,h()-65);
		box->color(FL_GRAY-1); 
		box->box(FL_FRAME_BOX);
		box->align(FL_ALIGN_CENTER|FL_ALIGN_INSIDE|FL_ALIGN_WRAP);
		box->label("Choose ASCII-format file with\n\
numbers\n\
arranged in columns and rows");
		box->labelsize(18);
		// Initialize the file chooser
		fc = new Fl_Native_File_Chooser();
		fc->filter("ASCII text files \t*.txt, *.TXT\n");			 
		end();
	}
};
/////////////////////////////////////////////////////////////////////
//////							DATA  DRAWING				/////////
//////		in a resizable double-buffered window			/////////
/////////////////////////////////////////////////////////////////////
class Draw_data : public Fl_Double_Window 
{
	
public:
  	Draw_data(int X,int Y,int W,int H) : Fl_Double_Window(X,Y,W,H) {}
	
	int handle(int event)
   {
		int ret = Fl_Double_Window::handle(event);
		switch (event)
      {
			case FL_ENTER: // the mouse pointer entered a widget
				ret = 1;   // FL_ENTER: must return(1) to receive FL_PUSH
				break;
         case FL_PUSH:  // will cause 'user damage' with a specific code
            if(Fl::event_button()==FL_LEFT_MOUSE) break;
            damage(FL_DAMAGE_USER1);
				ret = 1;   // FL_ENTER: must return(1) to receive FL_PUSH
				break;
		}
		return(ret);
	}
   //  damage(mask) indicates that a partial update of the object is needed
	void draw_circle() // Draw a circle at mouse pointer location
   {      
      int xm = (int)Fl::event_x();
      int ym = (int)Fl::event_y();
		
		fl_color(pt_color);
		fl_pie(xm-pt_size/2, ym-pt_size/2, pt_size, pt_size, 0.0, 360.0);
	}			
	
	void draw(void)  // Function draw() within class Draw_seis
	{
		// User damage ONLY? - just do this and done	
		if ( damage() == FL_DAMAGE_USER1 ) 
		{ 
			draw_circle();
			return; 
		}
			
		Fl_Double_Window::draw();
		/**   don't declare reserved 'x' and 'y' variables !!!   **/
		int i,j,sta;
		int xx, yy, x0, y0, x_dim, y_dim;
		float valx,valy,dx,dy;
				
		x_dim = w() - 2*MARGIN;  y_dim = h() - 2*MARGIN;
		//cout << "Xdim = " << x_dim << ", Y_dim = " << y_dim << endl;

		fl_rectf(0,0,w(),h(),235,235,235);
		fl_color(0); fl_rect(0,0,w(),h());
		fl_rectf(MARGIN,MARGIN,x_dim,y_dim,255,255,255);
		// FL_COLOR:   0==BLACK, 1==RED, 2==GREEN, 7==WHITE
		fl_color(0);	fl_font(1,12);	// Helvetica, 12 pixel
		dx = max_x-min_x;  dy = max_y-min_y;
				
		//  Draw tick marks and labels on the X-axis
		for(valx=min_x;valx<=max_x;valx+=xticks)
		{
			xx = MARGIN + (int)((valx-min_x)*x_dim/dx);
			yy = MARGIN-20;
			fl_color(23); fl_yxline(xx, MARGIN, h()-MARGIN); // gray vertical line
			fl_color(0); fl_yxline(xx, MARGIN-6, MARGIN);	 // vertical line
			sprintf(cgbuf,"%g",valx);	fl_draw(cgbuf,xx-8,MARGIN-12);
		}
		
		//  Draw tick marks and labels on the Y-axis
		for(valy=min_y;valy<=max_y;valy+=yticks)
		{
			if(reverseY==0) yy = MARGIN + (int)((valy-min_y)*y_dim/dy);
			else yy = MARGIN + y_dim - (int)((valy-min_y)*y_dim/dy); 
			fl_color(23); fl_xyline(MARGIN, yy, w()-MARGIN); // gray horiz line
			fl_color(0); fl_xyline(MARGIN-6, yy, MARGIN);	 // horiz line
			sprintf(cgbuf,"%g",valy);	fl_draw(cgbuf,RIM+25,yy+4);
		}		
			
		//		Draw data points with fl_circle,
		//    optionally, connect points with lines
		  
		fl_color(pt_color);	
		// fl_line_style(0,pt_size); // to use with fl_circle
		sta=0;
		for(i=0;i<Ndat;i++)
		{
			valx = logs[Ncur*i+curve_x];  valy = logs[Ncur*i+curve_y];
			if(valx<min_x || valx>max_x) { sta=0; continue; }
			if(valy<min_y || valy>max_y) { sta=0; continue; }
				
			xx = MARGIN + (int)((valx-min_x)*x_dim/dx);
			if(reverseY==0) yy = MARGIN + (int)((valy-min_y)*y_dim/dy);
			else yy = MARGIN + y_dim - (int)((valy-min_y)*y_dim/dy);
			
			//fl_circle((double)xx,(double)yy,(double)pt_size/2.0);
			fl_pie(xx-pt_size/2, yy-pt_size/2, pt_size, pt_size, 0.0, 360.0);
			
			if(line_draw==1 && sta>0) fl_line(x0,y0,xx,yy);
			x0 = xx;		y0 = yy;   sta=1;
		}
		
		fl_line_style(0,1); // solid, 1 pixel wide
		fl_color(0);	fl_rect(MARGIN,MARGIN,x_dim,y_dim);
		fl_font(1,13);  fl_draw(Xtext_hdr,2*x_dim/3,40);

		//  Rotates the text string 90˚ counter-clockwise
		fl_draw(90,Ytext_hdr,RIM+15,2*y_dim/3);
		
	} 	// void draw(void)
	
} *data_plot;
//
////////////////////////////////////////////////////////////////
/************    List of Call Back functions    ***************/
////////////////////////////////////////////////////////////////

void input_curve_x(Fl_Widget*,void*data) {
	Fl_Input_Choice *in_curve_x=(Fl_Input_Choice *)data;

	int j,no=-999;

	for(j=0;j<Ncur;j++)
		if(strncmp(curve[j],in_curve_x->value(),3) == 0) no=j;
			
	if(no>-999)
	{ 
		curve_x = no;
		min_x = mini[no];  max_x = maxi[no];
		sprintf(cgbuf," %g",min_x);  min_x_inp->value(cgbuf);
		sprintf(cgbuf," %g",max_x);  max_x_inp->value(cgbuf);
	}
}
void input_curve_y(Fl_Widget*,void*data) {
	Fl_Input_Choice *in_curve_y=(Fl_Input_Choice *)data;
	
	int j,no=-999;

	for(j=0;j<Ncur;j++)
		if(strncmp(curve[j],in_curve_y->value(),3) == 0) no=j;
	
	if(no>-999)
	{ 
		curve_y = no;
		min_y = mini[no];  max_y = maxi[no];
		sprintf(cgbuf," %g",min_y);  min_y_inp->value(cgbuf);
		sprintf(cgbuf," %g",max_y);  max_y_inp->value(cgbuf);
	}
}
//
void input_min_x(Fl_Widget *, void *)
{
	float val = atof(min_x_inp->value());
	float max = maxi[curve_x];
	if(val>max) val = max;

	min_x = val;
	sprintf(cgbuf," %g",val);  min_x_inp->value(cgbuf);
}
void input_max_x(Fl_Widget *, void *) 
{
	float val = atof(max_x_inp->value());
	float min = mini[curve_x];
	if(val<min) val = min;

	max_x = val;
	sprintf(cgbuf," %g",val);  max_x_inp->value(cgbuf);
}
void input_min_y(Fl_Widget *, void *) 
{
	float val = atof(min_y_inp->value());
	float max = maxi[curve_y];
	if(val>max) val = max;
	
	min_y = val;
	sprintf(cgbuf," %g",val);  min_y_inp->value(cgbuf);
}
void input_max_y(Fl_Widget *, void *) 
{
	float val = atof(max_y_inp->value());
	float min = mini[curve_y];
	if(val<min) val = min;
	
	max_y = val;
	sprintf(cgbuf," %g",val);  max_y_inp->value(cgbuf);
}
//
void input_xtick(Fl_Widget *, void *) {
	xticks = atof(xtick_inp->value());
}
void input_ytick(Fl_Widget *, void *) {
	yticks = atof(ytick_inp->value());
}
//
void spinner_cb(Fl_Widget *w, void *) {
   Fl_Spinner *s = (Fl_Spinner *)w;	
   pt_size = (int)s->value();
}
//
void Xhdr_cb(Fl_Widget *w, void *) {
   Fl_Input *s = (Fl_Input *)w;	
   Xtext_hdr = s->value();
}
void Yhdr_cb(Fl_Widget *w, void *) {
   Fl_Input *s = (Fl_Input *)w;	
   Ytext_hdr = s->value();
}
//
void connect_cb(Fl_Widget *w, void *) {
   Fl_Check_Button *check = (Fl_Check_Button *)w;
   line_draw = check->value();
}
void reverse(Fl_Widget *w, void *) {
   Fl_Check_Button *check = (Fl_Check_Button *)w;
   reverseY = check->value();
}
///////    Color Chooser    ////////////
void do_color(Fl_Widget*, void*) {	
	pt_color = fl_show_colormap(bg_color);
}
//
/////////////////////////////////////////////////////////////////////
///////////////           DRAWING Window          ///////////////////
/////////////////////////////////////////////////////////////////////
void doPLOT(Fl_Widget *, void*)
{  
	
	// Define Window Size with Margins
	win_w = Fl::w()/4;
	win_h = Fl::h()*2/5;

	draw_window = new Fl_Double_Window(500,200,win_w,win_h,
											"Data Plot Window");
	data_plot = new Draw_data(0,0,win_w,win_h);
	
	draw_window->end();
	draw_window->resizable(draw_window);
	draw_window->show();
}
///////////    EXIT    ////////////
void exit_b(Fl_Widget *, void *) {
	exit(0);
}

/////////////////////////////////////////////////////////////////////
/****   MAIN   MAIN   MAIN   MAIN   MAIN   MAIN   MAIN   MAIN   ****/
/////////////////////////////////////////////////////////////////////
int main(int argc, char** argv)
{
	int i,cur;
	double t,val;
	char cbuf1[90],cbuf2[90],cbuf3[90],cbuf4[90],cbuf[90];	
	///////////////////////////////////////////////////////////
	
	Fl::scheme(NULL); //  "plastic" for Aqua and "gtk+"
	// NULL use the scheme defined in the FLTK_SCHEME env.var. 
	
	if(argc == 2) strcpy(inpname,argv[1]);		// Do not use terminal
	else	// if you want to create an Apple  *.app bundle application
	{
		File_menu *choice = new File_menu();
		choice->show();
		choice->end(); // no more widgets will be added to the window
		while(chosen<0) Fl::wait();
		choice->hide();
	}

	read_file_info(inpname);

	/////////////////////////////////////////////////////////////////
	
	opts_window = new Fl_Window(1050,50,420,700,"Display Manager");
	// The follwing widgets belong to this window	

	Fl_Box* label1 = new Fl_Box(FL_FRAME_BOX,20,10,380,70,""); label1->color(6);
	
	sprintf(cbuf4,"  File Information  ");	
	Fl_Box* label2 = new Fl_Box(80,20,250,20,cbuf4);
	label2->labelcolor(FL_BLACK);
	label2->labelfont(FL_HELVETICA_BOLD);	label2->labelsize(16);
		
	sprintf(cbuf1,"Input file contains %d curves, %d samples each",Ncur,Ndat);
	Fl_Box* label3 = new Fl_Box(80,45,250,20,cbuf1);
	label3->labelfont(FL_HELVETICA);	label3->labelsize(14);

	/////////////////////////////////////////////////////////////////

	Fl_Input_Choice in_curve_x(320,100,60,25,
							"Assign curve (column #) to the X-axis:  ");
	in_curve_x.callback(input_curve_x, (void*)&in_curve_x);	
	for(i=0;i<Ncur;i++) in_curve_x.add(curve[i]);
	in_curve_x.value(curve[curve_x]);		// Default value = 0
	in_curve_x.labelsize(15); in_curve_x.labelfont(FL_HELVETICA_BOLD);
	in_curve_x.align(Fl_Align(FL_ALIGN_LEFT));
	
	Fl_Box* label4 = new Fl_Box(50,135,100,30,"Range of values :");
	label4->labelfont(FL_HELVETICA_BOLD); label4->labelsize(15);
	min_x_inp = new Fl_Float_Input(205, 138, 75, 25, " from ");
	sprintf(cbuf," %g",min_x);				min_x_inp->value(cbuf); 
	min_x_inp->callback(input_min_x);	min_x_inp->labelfont(0);
	max_x_inp = new Fl_Float_Input(305, 138, 75, 25, " to ");
	sprintf(cbuf," %g",max_x);				max_x_inp->value(cbuf);   
	max_x_inp->callback(input_max_x);	max_x_inp->labelfont(0);
	
	xtick_inp = new Fl_Float_Input(320, 170, 60, 25,
						"Interval between tick marks on the X-axis :");
	xtick_inp->labelfont(FL_HELVETICA_BOLD);	xtick_inp->labelsize(15);
	sprintf(cbuf," %g",xticks);					xtick_inp->value(cbuf);
	xtick_inp->callback(input_xtick);
	
	Fl_Box* line1 = new Fl_Box(FL_FRAME_BOX,15,205,385,5,""); line1->color(0);
	
	/////////////////////////////////////////////////////////////////

	Fl_Input_Choice in_curve_y(320,220,60,25,
							"Assign curve (column #) to the Y-axis:  ");
	in_curve_y.callback(input_curve_y, (void*)&in_curve_y);
	for(i=0;i<Ncur;i++) in_curve_y.add(curve[i]);
	in_curve_y.value(curve[curve_y]);		// Default value = 1
	in_curve_y.labelsize(15); in_curve_y.labelfont(FL_HELVETICA_BOLD);
	in_curve_y.align(Fl_Align(FL_ALIGN_LEFT));
	
	Fl_Box* label5 = new Fl_Box(50,255,100,30,"Range of values :");
	label5->labelfont(FL_HELVETICA_BOLD); label5->labelsize(15);
	min_y_inp = new Fl_Float_Input(205, 258, 75, 25, " from ");
	sprintf(cbuf," %g",min_y);				min_y_inp->value(cbuf);
	min_y_inp->callback(input_min_y);	min_y_inp->labelfont(0);
	max_y_inp = new Fl_Float_Input(305, 258, 75, 25, " to ");
	sprintf(cbuf," %g",max_y);				max_y_inp->value(cbuf);
	max_y_inp->callback(input_max_y);	max_y_inp->labelfont(0);
	
	ytick_inp = new Fl_Float_Input(320, 290, 60, 25,
						"Interval between tick marks on the Y-axis :");
	ytick_inp->labelfont(FL_HELVETICA_BOLD);	ytick_inp->labelsize(15);
	sprintf(cbuf," %g",yticks);					ytick_inp->value(cbuf);
	ytick_inp->callback(input_ytick);

	Fl_Box* line2 = new Fl_Box(FL_FRAME_BOX,15,325,385,5,""); line2->color(0);
	
	//////////////////////////////////////////////////////////////////

	Fl_Box* label6 = new Fl_Box(70,335,240,30,
									"Select plot parameters:");
	label6->labelfont(FL_HELVETICA_BOLD); label6->labelsize(15);
	
	
	Fl_Spinner* spin = new Fl_Spinner(125,380,50,25,"Point Size ");
	spin->labelsize(15);	spin->labelfont(FL_HELVETICA_BOLD);
	spin->minimum(1);		spin->maximum(10);		spin->step(1);
	spin->value(pt_size);	spin->align(Fl_Align(FL_ALIGN_LEFT));
	spin->callback(spinner_cb);
	Fl_Box* label14 = new Fl_Box(170,383,60,20,"pixel");
	label14->labelfont(FL_HELVETICA); label14->labelsize(15);

	choose_color = new Fl_Button(260,383,100,25,"Point Color ");
	choose_color->callback(do_color);	choose_color->labelcolor(FL_BLACK);
	choose_color->labelfont(3);			choose_color->labelsize(14);	
	choose_color->color(3);					choose_color->box(FL_RSHADOW_BOX);
	
	connected = new Fl_Check_Button(70,420,150,25,
							" Connect data points with line?");
	connected->labelfont(FL_HELVETICA_BOLD);	connected->labelsize(15);
	connected->value(line_draw);				connected->callback(connect_cb);
	

	plot_dir = new Fl_Check_Button(70,450,150,25,
									" Reverse Y-axis direction?");
	plot_dir->labelfont(FL_HELVETICA_BOLD);	plot_dir->labelsize(15);
	plot_dir->value(reverseY);				plot_dir->callback(reverse);
	
	
	Fl_Input* Xaxis_hdr = new Fl_Input(170,485,200,25,
							"Label the X-axis: ");
	Xaxis_hdr->value(Xtext_hdr);				Xaxis_hdr->callback(Xhdr_cb);
	Xaxis_hdr->labelfont(FL_HELVETICA_BOLD);	Xaxis_hdr->labelsize(15);
	
	Fl_Input* Yaxis_hdr = new Fl_Input(170,520,200,25,
											"Label the Y-axis: ");
	Yaxis_hdr->value(Ytext_hdr);				Yaxis_hdr->callback(Yhdr_cb);
	Yaxis_hdr->labelfont(FL_HELVETICA_BOLD);	Yaxis_hdr->labelsize(15);
	
	Fl_Box* line3 = new Fl_Box(FL_FRAME_BOX,15,560,3685,5,""); line3->color(0);
		
	//////////////////////////////////////////////////////////////////		
				
	apply = new Fl_Button(135,590,140,30,"Plot the data");
	apply->color(62);				apply->labelcolor(FL_BLACK);
	apply->labelfont(3);			apply->labelsize(16);
	apply->box(FL_RSHADOW_BOX);	apply->callback(doPLOT);
	
	/////////////////////////////////////////////////////////////////
		
	end_this = new Fl_Button(165,640, 80, 30, "Exit");
	end_this->color(201);		end_this->labelcolor(FL_BLACK);
	end_this->labelfont(3);		end_this->labelsize(18);	
	end_this->box(FL_RSHADOW_BOX);	end_this->callback(exit_b);	

	opts_window->end(); 
	opts_window->show();
	/////////////////////////////////////////////////////////////////

	return(Fl::run());
   	return 0;
}
//////////////////////////////////////////////////////////////////////
/****    END_MAIN   END_MAIN   ENF_MAIN   END_MAIN   END_MAIN    ****/
//////////////////////////////////////////////////////////////////////
/****               Called Function Definition                   ****/
//////////////////////////////////////////////////////////////////////
void read_file_info(const char *name) // Reads ALL log-data int logs[]
{
	int i,j,k,n,pass,counter;
	char *ptr, buf[80], bigbuf[1000], keys[]="1234567890-.";
	float val;

	if ((in_fp = fopen(name, "r")) == NULL)
	{fl_message("ERROR opening input file!\nClose the window to exit");exit(1);}
	
	fseek(in_fp, 0L, SEEK_SET);	// go to beginning
	
	// Test if the first string contains non-digits	and count number of columns
	fgets(bigbuf,999,in_fp); 
	ptr = strtok (bigbuf," ");  j=0;  
	while (ptr != NULL)
	{
		sscanf(ptr,"%s",buf);
		//if(isnumber((int)buf[0]) == 0)
		if(strcspn(buf,keys) ==1)
		{ fl_message("Input file contains non-decimal characters:\n\
%s\nClose the window to exit...",buf); fclose(in_fp); exit(1); }
		ptr = strtok (NULL, " ");	j++;	// j = column index
	}
	Ncur = j;

	fseek(in_fp, 0L, SEEK_SET);	// go to beginning and count number of lines
	Ndat=0;
	while(fgets(bigbuf,999,in_fp)) Ndat++;
	
	if(Ncur>=MAX_CURVE)
	{ fl_message("Number of curves exceeds defined maximum\n\
Close the window to exit"); fclose(in_fp); exit(1); }

	if(Ncur<2)
	{ fl_message("File must have 2+ columns\n\
Close the window to exit"); fclose(in_fp); exit(1); }

	mini = new (nothrow) float[Ncur];			// for logs scaling
	maxi = new (nothrow) float[Ncur];			// for logs scaling
	logs = new (nothrow) float[Ncur * Ndat];  // for log data  ~10 MB

	if(logs == 0)
	{ 
		fl_message("ERROR: NO memory to accomodate data!\n\
Close the window to exit...");	
		fclose(in_fp);  exit(1);
	}
	///////////////////////////////////////////////////////////////
	////////////////    READ  DATA  into  MEMORY   ////////////////
	///////////////////////////////////////////////////////////////
	
	fseek(in_fp, 0L, SEEK_SET);		// Go to beginning of the file	
	i=0;										// and
	while(fgets(bigbuf,999,in_fp))	// READ DATA into logs[..]
	{		
		ptr = strtok (bigbuf," ");  j=0;  
		while (ptr != NULL)
		{
			sscanf(ptr,"%f",logs+Ncur*i+j);
			ptr = strtok (NULL, " ");	j++;	// j = curve index
		}
		if(j != Ncur) 
		{ fl_message("Data loading ERROR: number of data columns varies\n\
Close the window to exit"); fclose(in_fp); exit(1); }
		i++;
		if(i>Ndat) 
		{ fl_message("Data loading ERROR: excessive number of lines\n\
Close the window to exit"); fclose(in_fp); exit(1); }			
	}
	fclose(in_fp);
	
	///////////////////////////////////////////////////////////////
	for(j=0;j<Ncur;j++)				// Search for Min-Max values 
	{
		sprintf(curve[j],"%3d",j+1);
	
		mini[j]=FLT_MAX;  maxi[j]=FLT_MIN;
		for(i=0;i<Ndat;i++)
		{
			val=logs[Ncur*i+j];
			{ if(val>maxi[j]) maxi[j]=val; if(val<mini[j]) mini[j]=val; }
		}
	}
	
	min_x = mini[curve_x];	min_y = mini[curve_y];
	max_x = maxi[curve_x];	max_y = maxi[curve_y];
	
}
